﻿Option Explicit On
Option Strict On
Imports System.Data
Imports System.Data.SqlClient
Imports System.IO

Public Class frmDataTables
    Private dtbClientes As DataTable
    Private dtSchema As DataTable
    Private strPath As String
    Private strArquivoIE As String = "D:\Arquivos de programas\Internet Explorer\Iexplore.exe"
    Private blnIsCarregando As Boolean
    Private intRows As Integer

    '****************************************
    'Carregando DataTable e Exibindo rotinas
    '****************************************

    Private Sub frmDataTables_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        'Carrega e exibe o DataTable padrão
        strPath = Application.StartupPath + "\"
        carregaDoDB(False)
        salvaArquivosXML(False)
        'Verifica a localização do IE
        If File.Exists(strArquivoIE) Then
            btnExibirDados.Visible = True
            btnExibirDataSet.Visible = True
            btnExibirDiffGram.Visible = True
        End If
    End Sub

    Private Sub btnRecarregaBD_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnRecarregaBD.Click
        'Re-Carrega o DataTable com namespace opcional
        carregaDoDB(chkIncluiNS.Checked)
        salvaArquivosXML(True)
    End Sub

    Private Sub carregaDoDB(ByVal blnWithNamespace As Boolean)
        'Carrega e exibe o DataTable com ou sem uma tabela namespace
        Dim strConn As String = My.Settings.connSQL
        Dim cnnCadastro As SqlConnection = New SqlConnection(strConn)
        Try
            Dim cmdClientes As SqlCommand = cnnCadastro.CreateCommand
            With cmdClientes
                .CommandType = CommandType.Text
                .CommandText = "SELECT * FROM Clientes"
            End With
            cnnCadastro.Open()
            Dim drClientes As SqlDataReader = cmdClientes.ExecuteReader(CommandBehavior.KeyInfo)
            dtbClientes = New DataTable
            dtSchema = drClientes.GetSchemaTable
            With dtbClientes
                .TableName = "Clientes"
                If blnWithNamespace Then
                    'Uncomment the following line to view effect on data and schemas
                    '.Prefix = "clientes"
                    .Namespace = "http://www.macoratti.net/schemas/Clientes"
                End If

                'Carrega os dados e aceita as alterações
                .Load(drClientes)
                .AcceptChanges()

                If .PrimaryKey.Length = 0 Then
                    'Define a chave primaria se estiver faltando
                    Dim acolKeys(1) As DataColumn
                    acolKeys(0) = .Columns(0)
                    .PrimaryKey = acolKeys
                End If

                'Testa a propriedade DataSet
                If Not .DataSet Is Nothing Then
                    Dim strNome As String = .DataSet.DataSetName
                    MsgBox(strNome)
                End If
            End With
            drClientes.Close()

            'Testa o DataTableReader
            Dim dtrClientes As New DataTableReader(dtbClientes)
            intRows = 0
            While dtrClientes.Read
                intRows += 1
            End While
            dtrClientes.Close()

            carregaDataGridViews()
        Catch excDT As Exception
            MsgBox(excDT.Message + excDT.StackTrace, , "DataTable Load Exception")
        Finally
            cnnCadastro.Close()
        End Try
    End Sub

    Private Sub carregaDataGridViews()
        'Carrega e e formata os dados
        blnIsCarregando = True
        With dgvClientes
            .DataSource = dtbClientes
            .AutoGenerateColumns = True
            .AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.DisplayedCells
            .Columns(0).Width = 70
            .Columns(7).Width = 70
            .RowHeadersWidth = 25
            .BorderStyle = BorderStyle.None
        End With
        'Carrega e formata o  schema
        With dgvSchema
            .DataSource = dtSchema
            .AutoGenerateColumns = True
            .AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.DisplayedCells
            .RowHeadersVisible = False
            .BorderStyle = BorderStyle.None
        End With
        blnIsCarregando = False
    End Sub
    '**********************************
    'Rotinas para Salvar e carregar XML
    '**********************************
    Private Sub salvaArquivosXML(ByVal blnShowMessage As Boolean)
        'Deleta e re-salva os dados, schema, e (se houve mudanças) os arquivos diffgram
        deletaArquivosXML()
        With dtbClientes
            .WriteXml(strPath + "Data.xml", System.Data.XmlWriteMode.IgnoreSchema)
            .WriteXml(strPath + "DataSet.xml", System.Data.XmlWriteMode.WriteSchema)
            .WriteXmlSchema(strPath + "Schema.xsd")
        End With
        btnExibirDados.Enabled = True
        btnExibirDataSet.Enabled = True
        btnExibirEsquema.Enabled = True
        'A propriedade HasChanges esta faltando
        Dim dtMudancas As New DataTable
        dtMudancas = dtbClientes.GetChanges
        Dim strMsg As String
        If dtMudancas Is Nothing Then
            strMsg = "Dados e esquema para " + intRows.ToString + " linhas escrita para '" + strPath + "' pasta."
            btnExibirDiffGram.Enabled = False
        Else
            dtMudancas.WriteXml(strPath + "Diffgram.xml", System.Data.XmlWriteMode.DiffGram)
            strMsg = "Dados para " + intRows.ToString + " linhas, schema, e alterações no diffgram escritas para '" + strPath + "' pasta e alterações aceitas."
            dtbClientes.AcceptChanges()
            btnExibirDiffGram.Enabled = True
        End If
        If blnShowMessage Then
            MsgBox(strMsg, , "Arquivos XMl Salvos")
        End If
        btnlerXML.Enabled = True
    End Sub

    Private Sub deletaArquivosXML()
        'Deleta todos so arquivos salvos
        File.Delete(strPath + "Data.xml")
        File.Delete(strPath + "DataSet.xml")
        File.Delete(strPath + "Diffgram.xml")
        File.Delete(strPath + "Schema.xsd")
        File.Delete(strPath + "Schema_app1.xsd")
        btnlerXML.Enabled = False
    End Sub

    Private Sub btnLerXML_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnlerXML.Click
        'Carrega ou tenta carregar o DataTable do DataSet salvo
        btnExibirDiffGram.Enabled = False
        Try
            dtbClientes = New DataTable
            With dtbClientes
                .ReadXml(strPath + "DataSet.xml")
                If File.Exists(strPath + "Diffgram.xml") Then
                    'aplica as alterações
                    .ReadXml(strPath + "Diffgram.xml")
                End If
                .AcceptChanges()
            End With

            'Testa o DataTableReader
            Dim dtrClientes As New DataTableReader(dtbClientes)
            dtSchema = dtrClientes.GetSchemaTable
            intRows = 0
            While dtrClientes.Read
                intRows += 1
            End While
            dtrClientes.Close()

            carregaDataGridViews()
        Catch excXML As Exception
            MsgBox(excXML.Message + excXML.StackTrace, , "Erro na leitura do DataTable : ReadXml")
        End Try
    End Sub

    '*****************************
    'Rotinas de edição do Grid View
    '*****************************

    Private Sub dgvClientes_CurrentCellChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles dgvClientes.CurrentCellChanged
        If Not blnIsCarregando Then
            'A propriedade HasChanges esta faltando
            Dim dtMudancas As New DataTable
            dtMudancas = dtbClientes.GetChanges
            If dtMudancas Is Nothing Then
                btnExibirDiffGram.Enabled = False
            Else
                dtMudancas.WriteXml(strPath + "Diffgram.xml", System.Data.XmlWriteMode.DiffGram)
                'dtbClientes.AcceptChanges()
                btnExibirDiffGram.Enabled = True
            End If
        End If
    End Sub

    Private Sub dgvClientes_DataError(ByVal sender As Object, ByVal e As System.Windows.Forms.DataGridViewDataErrorEventArgs) Handles dgvClientes.DataError
        'Manipula os erros de edição do grid
        MsgBox(e.Exception.Message + e.Exception.StackTrace, , "Erro na edição do DataTable")
    End Sub

    '****************************************
    'Exibe o tratamento de evento dos botões
    '*****************************************

    Private Sub btnExibirDados_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnExibirDados.Click
        exibirNoIE(strPath + "Data.xml")
    End Sub

    Private Sub btnExibirDataSet_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnExibirDataSet.Click
        exibirNoIE(strPath + "DataSet.xml")
    End Sub

    Private Sub btnExibirEsquema_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnExibirEsquema.Click
        exibirNoIE(strPath + "Schema.xsd")
    End Sub

    Private Sub btnExibirDiffGram_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnExibirDiffGram.Click
        exibirNoIE(strPath + "DiffGram.xml")
    End Sub

    Private Sub exibirNoIE(ByVal strArquivo As String)
        'System.Diagnostics.Process.Start(strArquivo)
        Shell("" + strArquivoIE + "" + strArquivo, AppWinStyle.NormalFocus)
    End Sub

End Class
